zone_label = "ether.edgerun.io/zone"
pod_type_label = "type"
function_label = "ether.edgerun.io/function"
api_gateway_type_label = 'api-gateway'
function_type_label = 'fn'
client_role_label = 'node-role.kubernetes.io/client'
worker_role_label = 'node-role.kubernetes.io/worker'

# Pod status constants
pod_not_running = 'Not Running'
pod_running = 'Running'
pod_unknown = "Unknown"
pod_failed = "Failed"
pod_succeeded = "Succeeded"
pod_pending = "Pending"
